use std::{env::current_dir, path::Path};

use cargo::{
    core::{Features, Workspace},
    util::toml_mut::{
        dependency::{PathSource, Source},
        manifest::LocalManifest,
    },
    GlobalContext,
};

fn main() -> anyhow::Result<()> {
    let cd = current_dir()?;
    let gctx = GlobalContext::default().unwrap();
    let feat = Features::default();

    // Assume we're running via `cargo run` and current_dir is a cargo project just for the sake of
    // example
    let ws = Workspace::new(&cd.join("Cargo.toml"), &gctx)?;
    let mut m = LocalManifest::try_new(&cd.join("Cargo.toml"))?;

    let d = m
        .get_dependency_versions("cargo", &ws, &feat)
        .collect::<Vec<_>>();

    for (_, dep) in d {
        let dep = dep?;

        // Errors because "../cargo is relative".
        // Path should be absolute, cargo then does its own
        // processing to make it relative to the Workspace.
        // Error doesn't actually happen until call to insert_into_table().
        let s = Source::Path(PathSource::new(Path::new("../cargo")));

        let dep = dep.clone().set_source(s);
        m.insert_into_table(&["dependencies".to_string()], &dep, &gctx, &cd, &feat)?;
    }
    Ok(())
}
